/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2009 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: , $
 * $Revision:  $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.sdk.mailmerge;

import com.sun.star.beans.NamedValue;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.text.MailMergeEvent;
import com.sun.star.uno.XComponentContext;
import com.sun.star.comp.helper.Bootstrap;
import com.sun.star.container.XNameAccess;
import com.sun.star.frame.XModel;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.sdb.XDocumentDataSource;
import com.sun.star.task.XJob;
import com.sun.star.text.XMailMergeBroadcaster;
import com.sun.star.text.XMailMergeListener;
import com.sun.star.text.XText;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.util.XCloseable;
import org.openoffice.sdk.tools.helpers.DatabaseHelper;
import org.openoffice.sdk.tools.helpers.Helper;
import org.openoffice.sdk.tools.helpers.TextHelper;
import org.openoffice.sdk.tools.helpers.UCBHelper;

/**
 *
 * @author ariel
 */
public class OOoMailMerge implements XMailMergeListener {

    private final XComponentContext m_xContext;
    private final String m_sDocBaseURL;
    private String m_sDatabaseDocURL;
    private final String m_sTabularDataDocURL;
    private String m_sDataSourceName;
    private final String m_sDataTableName;
    
    public OOoMailMerge(XComponentContext xContext) {
        this.m_xContext = xContext;
        this.m_sDocBaseURL = Helper.getURLFromRelativePath(m_xContext, "docs");
        this.m_sDataSourceName = "DataSource Demo";
        this.m_sDataTableName = "datasource_sample";
        this.m_sTabularDataDocURL = m_sDocBaseURL + "/datasources/" + m_sDataTableName + ".cvs";
        this.m_sDatabaseDocURL = m_sDocBaseURL + "/databases/database_sample.odb";
    }
    
    protected void runDemo(){
        // create the data we wiil use as source
        createTestTabularData(m_sTabularDataDocURL);
        
        // create and register a data source
        createDataSource(m_sTabularDataDocURL);

        // create the OOO Writer document
        String sTextDocURL = createTextDocWithFields(m_sDataSourceName, m_sDataTableName);

        // do the mail merge
        doMailMerge(sTextDocURL, m_sDataSourceName, m_sDataTableName);

        //cleanUp(sRegisteredDataSourceName);
    }
    
    private void createTestTabularData(String sURL) {
        try {
            if (!UCBHelper.getSFA(m_xContext).exists(sURL)) {
                String[] sData = {
                    "Treatment,First Name,Last Name,Street,City",
                    "Mr.,Homer,Simpson,Evergreen Avenue 555, Springfield",
                    "Mrs.,Marge,Simpson,Evergreen Avenue 555, Springfield",
                    "Miss,Lisa,Simpson,Evergreen Avenue 555, Springfield",
                    "Mr.,Bart,Simpson,Evergreen Avenue 555, Springfield",
                    "Miss,Marggie,Simpson,Evergreen Avenue 555, Springfield",};

                UCBHelper.writeLinesToFile(m_xContext, sURL, sData, false);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    protected void createDataSource(String sFlatSourceURL) {
        try {
            XPropertySet xDataSourceProps = null;

            XNameAccess xNameAccess = (XNameAccess) UnoRuntime.queryInterface(
                    XNameAccess.class, DatabaseHelper.getDatabaseContext(m_xContext));

            if (xNameAccess.hasByName(m_sDataSourceName)) {

                xDataSourceProps = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class, xNameAccess.getByName(m_sDataSourceName));

                XDocumentDataSource xDataSourceDocAccess = (XDocumentDataSource)
                        UnoRuntime.queryInterface(XDocumentDataSource.class,
                        xDataSourceProps);

                XModel xModel = (XModel) UnoRuntime.queryInterface(XModel.class,
                        xDataSourceDocAccess.getDatabaseDocument());
                m_sDatabaseDocURL = xModel.getURL();

            } else {
                m_sDatabaseDocURL = m_sDocBaseURL + "/databases";
                m_sDatabaseDocURL = Helper.createUniqueNameForFile(
                        m_xContext, m_sDatabaseDocURL, "database_sample", "odb");

                PropertyValue[] aDataSourceInfo = new PropertyValue[4];

                aDataSourceInfo[0] = new PropertyValue();
                aDataSourceInfo[0].Name = "Extension";
                aDataSourceInfo[0].Value = "cvs";

                aDataSourceInfo[1] = new PropertyValue();
                aDataSourceInfo[1].Name = "Charset";
                aDataSourceInfo[1].Value = "utf-8";

                aDataSourceInfo[2] = new PropertyValue();
                aDataSourceInfo[2].Name = "HeaderLine";
                aDataSourceInfo[2].Value = Boolean.TRUE;

                aDataSourceInfo[3] = new PropertyValue();
                aDataSourceInfo[3].Name = "FieldDelimiter";
                aDataSourceInfo[3].Value = ",";

                xDataSourceProps = DatabaseHelper.createDataSource(
                        m_xContext, m_sDatabaseDocURL,
                        DatabaseHelper.Protocols.PROTOCOL_FLAT + sFlatSourceURL,
                        aDataSourceInfo,
                        false, "", "",
                        true, m_sDataSourceName);
                if (xDataSourceProps != null) {
                    try {
                        m_sDataSourceName = AnyConverter.toString(
                                xDataSourceProps.getPropertyValue("Name"));

                        System.out.printf("\nRegistered under the name \"%s\"\n",
                                m_sDataSourceName);

                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String createTextDocWithFields(String sDataSourceName, String sTableName) {
        String sURL = null;
        try {
            XTextDocument xTextDocument = Helper.createNewWriterDoc(m_xContext);
            XMultiServiceFactory xDocFactory = (XMultiServiceFactory) UnoRuntime.queryInterface(
                    XMultiServiceFactory.class, xTextDocument);

            XText xDocText = xTextDocument.getText();
            XTextCursor xDocCursor = xDocText.createTextCursorByRange(xDocText.getStart());

            TextHelper.insertText(xDocCursor, "Dear ");
            //"Treatment,First Name,Last Name,Street,City"
            TextHelper.createDatabaseTextField(xDocFactory, xDocCursor,
                    sDataSourceName,
                    sTableName,
                    "Treatment");
            TextHelper.insertText(xDocCursor, " ");
            //"Treatment,First Name,Last Name,Street,City"
            TextHelper.createDatabaseTextField(xDocFactory, xDocCursor,
                    sDataSourceName,
                    sTableName,
                    "First Name");
            TextHelper.insertText(xDocCursor, " ");
            //"Treatment,First Name,Last Name,Street,City"
            TextHelper.createDatabaseTextField(xDocFactory, xDocCursor,
                    sDataSourceName,
                    sTableName,
                    "Last Name");
            TextHelper.insertText(xDocCursor, ",");
            TextHelper.insertParaBreak(xDocCursor);
            TextHelper.insertText(xDocCursor, "We want to thank you for " +
                    "all the joy that you gave to us.");

            sURL = m_sDocBaseURL + "/templates";
            sURL = Helper.createUniqueNameForFile(m_xContext, sURL, "template_sample", "ott");
            Helper.storeWriterDocAsTemplate(xTextDocument, sURL);
            sURL = xTextDocument.getURL();

            XCloseable xCloseable = (XCloseable) UnoRuntime.queryInterface(
                    XCloseable.class, xTextDocument);
            xCloseable.close(true);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return sURL;
        }
    }

    protected void doMailMerge(String sTextDocURL, String sDataSourceName, String sTableName){
        try {
            XPropertySet xMailMerge = (XPropertySet) Helper.createUnoService(
                    m_xContext, "com.sun.star.text.MailMerge", XPropertySet.class);

            NamedValue[] aJobArgs = new NamedValue[8];

            aJobArgs[0] = new NamedValue("DocumentURL", sTextDocURL);
            aJobArgs[1] = new NamedValue("DataSourceName", sDataSourceName);
            aJobArgs[2] = new NamedValue("CommandType", new Integer(com.sun.star.sdb.CommandType.TABLE));
            aJobArgs[3] = new NamedValue("Command", sTableName);
            aJobArgs[4] = new NamedValue("OutputType", new Short(com.sun.star.text.MailMergeType.FILE));
            aJobArgs[5] = new NamedValue("OutputURL", m_sDocBaseURL + "/output");
            aJobArgs[6] = new NamedValue("FileNamePrefix", "First Name");
            aJobArgs[7] = new NamedValue("FileNameFromColumn", new Boolean(true));

            XMailMergeBroadcaster xMailMergeBroadcaster = (XMailMergeBroadcaster)
                    UnoRuntime.queryInterface(XMailMergeBroadcaster.class, xMailMerge);
            xMailMergeBroadcaster.addMailMergeEventListener(this);

            XJob xJob = (XJob) UnoRuntime.queryInterface(XJob.class, xMailMergeBroadcaster);
            xJob.execute(aJobArgs);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    protected void cleanUp(String sRegisteredDSName){
        if (sRegisteredDSName != null && !sRegisteredDSName.isEmpty())
            DatabaseHelper.deregisterDataSource(m_xContext, sRegisteredDSName);
    }

    //**************************************************************************

    public void notifyMailMergeEvent(MailMergeEvent aMailMergeEvent) {
        System.out.println("notifyMailMergeEvent...");
        if (aMailMergeEvent.Model != null)
            System.out.println(aMailMergeEvent.Model.getURL() + '\n');

    }

    //*************************************************************************
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        int nStatus = 0;
        try {
            // get the remote office component context
            XComponentContext xContext = Bootstrap.bootstrap();
            if (xContext != null) {

               OOoMailMerge demo = new OOoMailMerge(xContext);
               demo.runDemo();

                System.out.println("Press 'Return' to finish the example!");
                java.io.BufferedReader reader = new java.io.BufferedReader(
                        new java.io.InputStreamReader(System.in));
                reader.read();
            } else {
                System.err.println("ERROR: Could not bootstrap default Office.");
                nStatus = 1;
            }
        }
        catch (java.lang.Exception e){
            e.printStackTrace();
            nStatus = 1;
        }
        finally {
            System.exit( nStatus );
        }
    }
    
}
