/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2009 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: , $
 * $Revision:  $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.sdk.tools.helpers;

import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XNameAccess;
import com.sun.star.frame.XStorable;
import com.sun.star.lang.XSingleServiceFactory;
import com.sun.star.sdb.XDocumentDataSource;
import com.sun.star.sdb.XOfficeDatabaseDocument;
import com.sun.star.sdbc.XConnection;
import com.sun.star.sdbc.XDataSource;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uno.XNamingService;


/**
 *
 * @author ariel
 */
public class DatabaseHelper {

    public interface Protocols {
        String PROTOCOL_DBASE = "sdbc:dbase:";
        String PROTOCL_EMBEDDED_HSQLDB = "sdbc:embedded:hsqldb";
        String PROTOCOL_FLAT = "sdbc:flat:";
    }

    public static Object getDatabaseContext(XComponentContext xContext){
        return Helper.createUnoService(xContext, "com.sun.star.sdb.DatabaseContext");
    }

    public static XPropertySet createDataSource(
                                                    XComponentContext xContext,
                                                    String sDatabaseDocURL,
                                                    String sDatasourceURL,
                                                    PropertyValue[] aDataSourceInfo,
                                                    boolean bIsPasswordRequired,
                                                    String sUserName,
                                                    String sPassword,
                                                    boolean bRegister,
                                                    String sRegisterName){
        XPropertySet xDataSourceProps = null;
        try {
            XSingleServiceFactory xDBContextFactory = (XSingleServiceFactory) UnoRuntime.queryInterface(
                    XSingleServiceFactory.class, getDatabaseContext(xContext));

            xDataSourceProps = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xDBContextFactory.createInstance());

            xDataSourceProps.setPropertyValue("URL", sDatasourceURL);
            xDataSourceProps.setPropertyValue("Info", aDataSourceInfo);
            xDataSourceProps.setPropertyValue("IsPasswordRequired", new Boolean(bIsPasswordRequired));
            xDataSourceProps.setPropertyValue("User", sUserName);
            xDataSourceProps.setPropertyValue("Password", sPassword);

            XDocumentDataSource xDataSourceDocAccess = (XDocumentDataSource)
                    UnoRuntime.queryInterface(XDocumentDataSource.class, xDataSourceProps);

            XOfficeDatabaseDocument xODBDoc = xDataSourceDocAccess.getDatabaseDocument();

            XStorable xStorable = (XStorable) UnoRuntime.queryInterface(
                    XStorable.class, xODBDoc);

            xStorable.storeAsURL(sDatabaseDocURL, new PropertyValue[0]);

            if (bRegister) {
                XNamingService xNamingService = (XNamingService) UnoRuntime.queryInterface(
                        XNamingService.class, xDBContextFactory);
                XNameAccess xNameAccess = (XNameAccess) UnoRuntime.queryInterface(
                        XNameAccess.class, xDBContextFactory);

                sRegisterName = Helper.createUniqueNameForContainerElement(
                        xNameAccess, sRegisterName);

                xNamingService.registerObject(sRegisterName, xDataSourceProps);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xDataSourceProps;
        }
    }

    public static XConnection getConnection(XComponentContext xContext, String sDSName, String sUser, String sPassword){
        XConnection xRet = null;
        try {
            XNamingService xNamingService = (XNamingService) UnoRuntime.queryInterface(
                    XNamingService.class, getDatabaseContext(xContext));
            XNameAccess xNameAccess = (XNameAccess) UnoRuntime.queryInterface(
                    XNameAccess.class, xNamingService);

            if (xNameAccess.hasByName(sDSName)) {
                XDataSource xDataSource = (XDataSource) UnoRuntime.queryInterface(
                        XDataSource.class, xNameAccess.getByName(sDSName));
                xRet = xDataSource.getConnection(sUser, sPassword);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xRet;
        }
    }

    public static boolean deregisterDataSource(XComponentContext xContext, String sDSName) {
        boolean bRet = false;
        try {
            XNamingService xNamingService = (XNamingService) UnoRuntime.queryInterface(
                    XNamingService.class, getDatabaseContext(xContext));
            XNameAccess xNameAccess = (XNameAccess) UnoRuntime.queryInterface(
                    XNameAccess.class, xNamingService);

            if (xNameAccess.hasByName(sDSName)) {
                xNamingService.revokeObject(sDSName);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return bRet;
        }
    }
}
