/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2009 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: , $
 * $Revision:  $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.sdk.tools.helpers;

import com.sun.star.awt.Size;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XNameAccess;
import com.sun.star.drawing.XShape;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.sdb.CommandType;
import com.sun.star.text.ControlCharacter;
import com.sun.star.text.SizeType;
import com.sun.star.text.TextContentAnchorType;
import com.sun.star.text.XDependentTextField;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextFieldsSupplier;
import com.sun.star.text.XTextFrame;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;

/**
 *
 * @author ariel
 */
public class TextHelper {

    public static void insertText(XTextRange xTextRange, String sStr) {
        xTextRange.getText().insertString(xTextRange, sStr, false);
    }

    public static void insertParaBreak(XTextRange xTextRange) {
        try {
            xTextRange.getText().insertControlCharacter(
                    xTextRange, ControlCharacter.PARAGRAPH_BREAK, false);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void insertTextContent(XTextRange xTextRange, XTextContent xContent) {
        try {
            xTextRange.getText().insertTextContent(xTextRange, xContent, false);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void insertTextFrame(XMultiServiceFactory xDocFactory, XTextRange xTextRange) {

        try {
            XTextFrame xTextFrame = createTextFrame(xDocFactory,
                    12000, 4000, TextContentAnchorType.AT_PARAGRAPH, SizeType.MIN);
            insertTextContent(xTextRange, xTextFrame);

            XText xFrameText = xTextFrame.getText();
            insertText(xFrameText.getEnd(), "Some text inside the frame.");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static XTextFrame createTextFrame(
            XMultiServiceFactory xDocFactory,
            int nWidth, int nHeight, TextContentAnchorType aAnchorType, short nSizeType){
        XTextFrame xTextFrame = null;
        try {
            xTextFrame = (XTextFrame) UnoRuntime.queryInterface(
                    XTextFrame.class, xDocFactory.createInstance(
                        "com.sun.star.text.TextFrame"));

            XShape xShape = (XShape) UnoRuntime.queryInterface(XShape.class, xTextFrame);
            xShape.setSize(new Size(nWidth, nHeight));

            XPropertySet xFrameProps = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xTextFrame);
            xFrameProps.setPropertyValue("AnchorType", aAnchorType);
            xFrameProps.setPropertyValue("SizeType", new Short(nSizeType));

        } catch (Exception ex){
            ex.printStackTrace();
        } finally {
            return xTextFrame;
        }
    }

    /**
     *
     * @param xDocFactory
     * @param xTextRange
     * @param sDataBaseName
     * @param sDataTableName
     * @param sDataColumnName
     * @return
     */
    public static XDependentTextField createDatabaseTextField(
                                                XMultiServiceFactory xDocFactory,
                                                XTextRange xTextRange,
                                                String sDataBaseName,
                                                String sDataTableName,
                                                String sDataColumnName){
        XDependentTextField xDependentTextField = null;
        try {
            xDependentTextField = (XDependentTextField) UnoRuntime.queryInterface(
                    XDependentTextField.class,
                    xDocFactory.createInstance("com.sun.star.text.textfield.Database"));
            
            XPropertySet xPropertySet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xDependentTextField);
            xPropertySet.setPropertyValue("Content", '<' + sDataColumnName + '>');

            XPropertySet xFieldMaster = createFieldMasterDatabase(
                    xDocFactory, sDataBaseName, sDataTableName, sDataColumnName);
            xDependentTextField.attachTextFieldMaster(xFieldMaster);

            insertTextContent(xTextRange, xDependentTextField);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xDependentTextField;
        }
    }


    public static XPropertySet createFieldMasterDatabase(
                                                        XMultiServiceFactory xDocFactory,
                                                        String sDataBaseName,
                                                        String sDataTableName,
                                                        String sDataColumnName) {
        XPropertySet xFieldMaster = null;
        try {
            String sCreateServiceName = "com.sun.star.text.fieldmaster.Database";
            // issue: different names Database vs. DataBase
            String sFieldMasterName = "com.sun.star.text.fieldmaster.DataBase" +
                    "." + sDataBaseName +
                    "." + sDataTableName +
                    "." + sDataColumnName;

            XTextFieldsSupplier xTextFieldsSupplier = (XTextFieldsSupplier) UnoRuntime.queryInterface(
                    XTextFieldsSupplier.class, xDocFactory);
            XNameAccess xFieldMasterAccess = xTextFieldsSupplier.getTextFieldMasters();

            if (xFieldMasterAccess.hasByName(sFieldMasterName)) {
                xFieldMaster = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class,
                        xFieldMasterAccess.getByName(sFieldMasterName));
            } else {
                xFieldMaster = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class,
                        xDocFactory.createInstance(sCreateServiceName));

                if (xFieldMaster != null) {
                    xFieldMaster.setPropertyValue("DataBaseName", sDataBaseName);
                    xFieldMaster.setPropertyValue("DataCommandType", new Integer(CommandType.TABLE));
                    xFieldMaster.setPropertyValue("DataTableName", sDataTableName);
                    xFieldMaster.setPropertyValue("DataColumnName", sDataColumnName);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return xFieldMaster;
        }
    }

    public static XPropertySet createMasterSetExpression(
                                                        XMultiServiceFactory xDocFactory,
                                                        String sFiledName,
                                                        byte nChapNumLevel,
                                                        String sNumSeparator,
                                                        short nSubType) {
        XPropertySet xFieldMaster = null;
        try {
            String sServiceName = "com.sun.star.text.fieldmaster.SetExpression";
            String sFieldMasterName = sServiceName + "." + sFiledName;

            XTextFieldsSupplier xTextFieldsSupplier = (XTextFieldsSupplier) UnoRuntime.queryInterface(
                    XTextFieldsSupplier.class, xDocFactory);
            XNameAccess xFieldMasterAccess = xTextFieldsSupplier.getTextFieldMasters();

            if (xFieldMasterAccess.hasByName(sFieldMasterName)) {
                xFieldMaster = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class,
                        xFieldMasterAccess.getByName(sFieldMasterName));
            } else {
                xFieldMaster = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class,
                        xDocFactory.createInstance(sServiceName));

                if (xFieldMaster != null) {
                    xFieldMaster.setPropertyValue("Name", sFiledName);
                    xFieldMaster.setPropertyValue("ChapterNumberingLevel", nChapNumLevel);
                    xFieldMaster.setPropertyValue("NumberingSeparator",sNumSeparator);
                    xFieldMaster.setPropertyValue("SubType", nSubType);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return xFieldMaster;
        }
    }

    public static XDependentTextField createSetExpression(
                                                        XMultiServiceFactory xDocFactory,
                                                        XPropertySet xFieldMaster,
                                                        String content,
                                                        short subType,
                                                        int numberFormat,
                                                        short numberingType) {
        Object textfield = null;
        XDependentTextField xDependentTextField = null;
        try {
            textfield = xDocFactory.createInstance(
                    "com.sun.star.text.TextField.SetExpression");
            if (textfield != null) {
                XPropertySet xSetExpression = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class, textfield);
                xSetExpression.setPropertyValue("Content", content);
                xSetExpression.setPropertyValue("SubType", subType);
                xSetExpression.setPropertyValue("NumberFormat",
                        numberFormat);
                xSetExpression.setPropertyValue("NumberingType",
                        numberingType);

                xDependentTextField = (XDependentTextField) UnoRuntime.queryInterface(
                        XDependentTextField.class, textfield);
                xDependentTextField.attachTextFieldMaster(xFieldMaster);
            }
        } catch (com.sun.star.uno.Exception ex) {
            ex.printStackTrace();
        } finally {
            return xDependentTextField;
        }
    }
}
