/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2009 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: , $
 * $Revision:  $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.sdk.tools.helpers;

import com.sun.star.io.XActiveDataSource;
import com.sun.star.io.XInputStream;
import com.sun.star.io.XOutputStream;
import com.sun.star.io.XSeekable;
import com.sun.star.io.XTextOutputStream;
import com.sun.star.ucb.XSimpleFileAccess3;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 *
 * @author ariel
 */
public class UCBHelper {

    public static XSimpleFileAccess3 getSFA( XComponentContext xContext ) {
        XSimpleFileAccess3 xSFA = null;
        try {
            xSFA = (XSimpleFileAccess3) Helper.createUnoService(xContext,
                    "com.sun.star.ucb.SimpleFileAccess", XSimpleFileAccess3.class);
        } catch (java.lang.Exception ex) {
            ex.printStackTrace();
        }
        return xSFA;
    }



    public static boolean writeFile( XComponentContext xContext, String sSourceURL, String sTargetURL){
        boolean bStored = false;
        XOutputStream xOutputStream = null;
        XInputStream xInputStream = null;
        try {
            XSimpleFileAccess3 xSFA = getSFA(xContext);

            if (!xSFA.exists(sSourceURL))
                return false;
            if (xSFA.exists(sTargetURL))
                xSFA.kill(sTargetURL);

            xInputStream = xSFA.openFileRead(sSourceURL);
            xOutputStream = xSFA.openFileWrite(sTargetURL);

            bStored = writeFile(xOutputStream, xInputStream);

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            if (xInputStream!=null) {
                try {
                    xInputStream.closeInput();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            if (xOutputStream!=null) {
                try {
                    xOutputStream.closeOutput();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            return bStored;
        }
    }

    /**
     * You MUST close the output and the input!
     *
     * @param xOutputStream
     * @param xInputStream
     * @return
     */
    public static boolean writeFile(XOutputStream xOutStream, XInputStream xInStream){
        boolean bStored = false;
        try {
            long nAvailable = xInStream.available();
            if (nAvailable == 0) {
                XSeekable xSeekable = (XSeekable) UnoRuntime.queryInterface(
                        XSeekable.class, xInStream );
                if (xSeekable!=null){
                    nAvailable = xSeekable.getLength();
                }
            }
            if (nAvailable > 0) {
                byte[][] lData = new byte[1][];
                int n = xInStream.readBytes(lData, new Long(nAvailable).intValue());

                // com.sun.star.lang.DisposedException here
                // becuase the XStream created with XStorage.openStreamElement
                // get disposed in the remote bridge!
                xOutStream.writeBytes(lData[0]);
                xOutStream.flush();
                bStored = true;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return bStored;
        }
    }

    public static boolean writeLinesToFile(
                                            XComponentContext xContext,
                                            String sTargetURL,
                                            String[] sLines,
                                            boolean bAppend){
        boolean bStored = false;
        char[] CRLF = { '\r', '\n' };
        String sLineBreak = new String(CRLF);
        try {
            bStored = writeLinesToFile(xContext, sTargetURL, sLines,
                    sLineBreak, "utf-8", bAppend);

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return bStored;
        }
    }

    /**
     *
     * @param xContext
     * @param sTargetURL
     * @param sLines
     * @param sLineBreak    line break (CR, LF, or CR/LF)
     * @param sEncoding
     * @param bAppend
     * @return
     */
    public static boolean writeLinesToFile(
                                            XComponentContext xContext,
                                            String sTargetURL,
                                            String[] sLines,
                                            String sLineBreak,
                                            String sEncoding,
                                            boolean bAppend){
        boolean bStored = false;
        XOutputStream xOutputStream = null;
        try {
            XSimpleFileAccess3 xSFA = getSFA(xContext);

            if (!bAppend && xSFA.exists(sTargetURL))
                xSFA.kill(sTargetURL);

            xOutputStream = xSFA.openFileWrite(sTargetURL);

            bStored = writeLinesToStream(xContext, xOutputStream, sLines,
                    sLineBreak, sEncoding, bAppend);

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            if (xOutputStream!=null) {
                try {
                    xOutputStream.closeOutput();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            return bStored;
        }
    }


    /**
     * @param xContext
     * @param xOutputStream
     * @param sLines
     * @param sLineBreak
     * @param sEncoding
     * @param bAppend
     * @return
     */
    public static boolean writeLinesToStream(
                                            XComponentContext xContext,
                                            XOutputStream xOutputStream,
                                            String[] sLines,
                                            String sLineBreak,
                                            String sEncoding,
                                            boolean bAppend){
        boolean bStored = false;
        try {
            if (bAppend) {
                XSeekable xSeekable = (XSeekable) UnoRuntime.queryInterface(
                        XSeekable.class, xOutputStream );
                if (xSeekable != null) {
                    long length = xSeekable.getLength();
                    xSeekable.seek(length);
                }
            }

            XTextOutputStream xTextOutputStream =
                    (XTextOutputStream) UnoRuntime.queryInterface(
                    XTextOutputStream.class,
                    xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.io.TextOutputStream", xContext));
            xTextOutputStream.setEncoding(sEncoding);

            XActiveDataSource xActiveDataSource =
                    (XActiveDataSource) UnoRuntime.queryInterface(
                    XActiveDataSource.class, xTextOutputStream);

            xActiveDataSource.setOutputStream(xOutputStream);

            for (String line : sLines) {
                xTextOutputStream.writeString(line + sLineBreak);
            }
            xTextOutputStream.flush();
            bStored = true;

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return bStored;
        }
    }
}
